@@ -6,7 +6,6 @@
 import argparse
 import concurrent.futures
 import copy
-import io
 import itertools
 import json
 import logging
@@ -18,12 +17,14 @@ import subprocess
 import sys
 import threading
 from datetime import datetime
+from functools import lru_cache
 
 import hglib
+import lmdb
 from tqdm import tqdm
 
-from bugbug import db, utils
-from bugbug.utils import LMDBDict, get_hgmo_patch
+from bugbug import db, rust_code_analysis_server, utils
+from bugbug.utils import LMDBDict
 
 logger = logging.getLogger(__name__)
 
@@ -32,11 +33,12 @@ hg_servers_lock = threading.Lock()
 thread_local = threading.local()
 
 COMMITS_DB = "data/commits.json"
+COMMIT_EXPERIENCES_DB = "commit_experiences.lmdb.tar.zst"
 db.register(
     COMMITS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_commits.latest/artifacts/public/commits.json.zst",
-    9,
-    ["commit_experiences.lmdb.tar.zst"],
+    11,
+    [COMMIT_EXPERIENCES_DB],
 )
 
 path_to_component = None
@@ -138,31 +140,31 @@ class Commit:
         self.minimum_test_file_size = 0
         self.test_files_modified_num = 0
         self.average_cyclomatic = 0.0
-        self.average_halstead_operands = 0.0
-        self.average_halstead_unique_operands = 0.0
-        self.average_halstead_operators = 0.0
-        self.average_halstead_unique_operators = 0.0
+        self.average_halstead_N1 = 0.0
+        self.average_halstead_n1 = 0.0
+        self.average_halstead_N2 = 0.0
+        self.average_halstead_n2 = 0.0
         self.average_source_loc = 0.0
         self.average_logical_loc = 0.0
         self.maximum_cyclomatic = 0
-        self.maximum_halstead_operands = 0
-        self.maximum_halstead_unique_operands = 0
-        self.maximum_halstead_operators = 0
-        self.maximum_halstead_unique_operators = 0
+        self.maximum_halstead_N2 = 0
+        self.maximum_halstead_n2 = 0
+        self.maximum_halstead_N1 = 0
+        self.maximum_halstead_n1 = 0
         self.maximum_source_loc = 0
         self.maximum_logical_loc = 0
         self.minimum_cyclomatic = sys.maxsize
-        self.minimum_halstead_operands = sys.maxsize
-        self.minimum_halstead_unique_operands = sys.maxsize
-        self.minimum_halstead_operators = sys.maxsize
-        self.minimum_halstead_unique_operators = sys.maxsize
+        self.minimum_halstead_N1 = sys.maxsize
+        self.minimum_halstead_n1 = sys.maxsize
+        self.minimum_halstead_N2 = sys.maxsize
+        self.minimum_halstead_n2 = sys.maxsize
         self.minimum_source_loc = sys.maxsize
         self.minimum_logical_loc = sys.maxsize
         self.total_cyclomatic = 0
-        self.total_halstead_operands = 0
-        self.total_halstead_unique_operands = 0
-        self.total_halstead_operators = 0
-        self.total_halstead_unique_operators = 0
+        self.total_halstead_N1 = 0
+        self.total_halstead_n1 = 0
+        self.total_halstead_N2 = 0
+        self.total_halstead_n2 = 0
         self.total_source_loc = 0
         self.total_logical_loc = 0
 
@@ -177,7 +179,11 @@ class Commit:
         self.files = files
         self.file_copies = file_copies
         self.components = list(
-            set(path_to_component[path] for path in files if path in path_to_component)
+            set(
+                path_to_component[path.encode("utf-8")].tobytes().decode("utf-8")
+                for path in files
+                if path.encode("utf-8") in path_to_component
+            )
         )
         self.directories = get_directories(files)
         return self
@@ -221,14 +227,15 @@ def get_commits():
     return db.read(COMMITS_DB)
 
 
-def _init(repo_dir):
-    global HG
-    os.chdir(repo_dir)
-    HG = hglib.open(".")
+def _init_process(repo_dir):
+    global HG, REPO_DIR
+    REPO_DIR = repo_dir
+    HG = hglib.open(REPO_DIR)
+    get_component_mapping()
 
 
-def _init_thread():
-    hg_server = hglib.open(".")
+def _init_thread(repo_dir):
+    hg_server = hglib.open(repo_dir)
     thread_local.hg = hg_server
     with hg_servers_lock:
         hg_servers.append(hg_server)
@@ -276,27 +283,37 @@ def hg_modified_files(hg, commit):
         file_copies[sys.intern(orig)] = sys.intern(copied)
 
     commit.set_files(
-        [sys.intern(f) for f in files_str.decode("utf-8").split("|")], file_copies
+        [sys.intern(f) for f in files_str.decode("utf-8").split("|") if f], file_copies
     )
 
 
-def get_touched_functions(path, deleted_lines, added_lines, content):
-    if content is None:
-        return set()
+def get_functions_from_metrics(metrics_space):
+    functions = []
 
-    function_data = code_analysis_server.function(path, content)
-    if not function_data:
-        return set()
+    if (
+        metrics_space["kind"] == "function"
+        and metrics_space["name"]
+        and metrics_space["name"] != "<anonymous>"
+    ):
+        functions.append(
+            {
+                "end_line": metrics_space["end_line"],
+                "name": metrics_space["name"],
+                "start_line": metrics_space["start_line"],
+            }
+        )
 
+    for space in metrics_space["spaces"]:
+        functions += get_functions_from_metrics(space)
+
+    return functions
+
+
+def get_touched_functions(metrics_space, deleted_lines, added_lines):
     touched_functions = set()
     touched_function_names = set()
 
-    functions = function_data["spans"]
-    functions = [
-        function
-        for function in functions
-        if not function["error"] and function["name"] != "<anonymous>"
-    ]
+    functions = get_functions_from_metrics(metrics_space)
 
     def get_touched(functions, lines):
         last_f = 0
@@ -347,34 +364,33 @@ def get_touched_functions(path, deleted_lines, added_lines, content):
 
 
 def get_metrics(commit, metrics_space):
-    if metrics_space["kind"] == "function":
+    name = metrics_space["name"]
+    error = metrics_space["kind"] in {"unit", "function"} and name == ""
+
+    if metrics_space["kind"] == "function" and not error:
         metrics = metrics_space["metrics"]
         commit.total_cyclomatic += metrics["cyclomatic"]
-        commit.total_halstead_unique_operands += metrics["halstead"]["unique_operands"]
-        commit.total_halstead_operands += metrics["halstead"]["operands"]
-        commit.total_halstead_unique_operators += metrics["halstead"][
-            "unique_operators"
-        ]
-        commit.total_halstead_operators += metrics["halstead"]["operators"]
+        commit.total_halstead_n2 += metrics["halstead"]["n2"]
+        commit.total_halstead_N2 += metrics["halstead"]["N2"]
+        commit.total_halstead_n1 += metrics["halstead"]["n1"]
+        commit.total_halstead_N1 += metrics["halstead"]["N1"]
         commit.total_source_loc += metrics["loc"]["sloc"]
         commit.total_logical_loc += metrics["loc"]["lloc"]
 
         commit.maximum_cyclomatic = max(
             commit.maximum_cyclomatic, metrics["cyclomatic"]
         )
-        commit.maximum_halstead_unique_operands = max(
-            commit.maximum_halstead_unique_operands,
-            metrics["halstead"]["unique_operands"],
+        commit.maximum_halstead_n2 = max(
+            commit.maximum_halstead_n2, metrics["halstead"]["n2"],
         )
-        commit.maximum_halstead_operands = max(
-            metrics["halstead"]["operands"], commit.maximum_halstead_operands
+        commit.maximum_halstead_N2 = max(
+            metrics["halstead"]["N2"], commit.maximum_halstead_N2
         )
-        commit.maximum_halstead_unique_operators = max(
-            metrics["halstead"]["unique_operators"],
-            commit.maximum_halstead_unique_operators,
+        commit.maximum_halstead_n1 = max(
+            metrics["halstead"]["n1"], commit.maximum_halstead_n1,
         )
-        commit.maximum_halstead_operators = max(
-            metrics["halstead"]["operators"], commit.maximum_halstead_operators
+        commit.maximum_halstead_N1 = max(
+            metrics["halstead"]["N1"], commit.maximum_halstead_N1
         )
         commit.maximum_source_loc = max(
             metrics["loc"]["sloc"], commit.maximum_source_loc
@@ -386,19 +402,17 @@ def get_metrics(commit, metrics_space):
         commit.minimum_cyclomatic = min(
             commit.minimum_cyclomatic, metrics["cyclomatic"]
         )
-        commit.minimum_halstead_unique_operands = min(
-            commit.minimum_halstead_unique_operands,
-            metrics["halstead"]["unique_operands"],
+        commit.minimum_halstead_n2 = min(
+            commit.minimum_halstead_n2, metrics["halstead"]["n2"],
         )
-        commit.minimum_halstead_operands = min(
-            metrics["halstead"]["operands"], commit.minimum_halstead_operands
+        commit.minimum_halstead_N2 = min(
+            metrics["halstead"]["N2"], commit.minimum_halstead_N2
         )
-        commit.minimum_halstead_unique_operators = min(
-            metrics["halstead"]["unique_operators"],
-            commit.minimum_halstead_unique_operators,
+        commit.minimum_halstead_n1 = min(
+            metrics["halstead"]["n1"], commit.minimum_halstead_n1,
         )
-        commit.minimum_halstead_operators = min(
-            metrics["halstead"]["operators"], commit.minimum_halstead_operators
+        commit.minimum_halstead_N1 = min(
+            metrics["halstead"]["N1"], commit.minimum_halstead_N1
         )
         commit.minimum_source_loc = min(
             metrics["loc"]["sloc"], commit.minimum_source_loc
@@ -408,11 +422,13 @@ def get_metrics(commit, metrics_space):
         )
 
     for space in metrics_space["spaces"]:
-        get_metrics(commit, space)
+        error |= get_metrics(commit, space)
+
+    return error
 
 
-def _transform(commit):
-    hg_modified_files(HG, commit)
+def transform(hg, repo_dir, commit):
+    hg_modified_files(hg, commit)
 
     if commit.ignored:
         return commit
@@ -422,7 +438,7 @@ def _transform(commit):
     test_sizes = []
     metrics_file_count = 0
 
-    patch = HG.export(revs=[commit.node.encode("ascii")], git=True)
+    patch = hg.export(revs=[commit.node.encode("ascii")], git=True)
     patch_data = rs_parsepatch.get_lines(patch)
     for stats in patch_data:
         path = stats["filename"]
@@ -436,7 +452,10 @@ def _transform(commit):
         after = None
         if not stats["deleted"]:
             try:
-                after = HG.cat([path.encode("utf-8")], rev=commit.node.encode("ascii"))
+                after = hg.cat(
+                    [os.path.join(repo_dir, path).encode("utf-8")],
+                    rev=commit.node.encode("ascii"),
+                )
                 size = after.count(b"\n")
             except hglib.error.CommandError as e:
                 if b"no such file in rev" not in e.err:
@@ -463,21 +482,38 @@ def _transform(commit):
         elif type_ in SOURCE_CODE_TYPES_TO_EXT:
             commit.source_code_files_modified_num += 1
 
-            touched_functions = get_touched_functions(
-                path, stats["deleted_lines"], stats["added_lines"], after
-            )
-            if len(touched_functions) > 0:
-                commit.functions[path] = list(touched_functions)
-
             commit.source_code_added += len(stats["added_lines"])
             commit.source_code_deleted += len(stats["deleted_lines"])
 
             if size is not None:
                 source_code_sizes.append(size)
-                metrics = code_analysis_server.metrics(path, after, unit=False)
-                if metrics.get("spaces"):
-                    metrics_file_count += 1
-                    get_metrics(commit, metrics["spaces"])
+
+                if type_ != "IDL/IPDL/WebIDL":
+                    metrics = code_analysis_server.metrics(path, after, unit=False)
+                    if metrics.get("spaces"):
+                        metrics_file_count += 1
+                        error = get_metrics(commit, metrics["spaces"])
+                        if error:
+                            logger.debug(
+                                f"rust-code-analysis error on commit {commit.node}, path {path}"
+                            )
+
+                        touched_functions = get_touched_functions(
+                            metrics["spaces"],
+                            stats["deleted_lines"],
+                            stats["added_lines"],
+                        )
+                        if len(touched_functions) > 0:
+                            commit.functions[path] = list(touched_functions)
+
+                    # Add "Objective-C/C++" type if rust-code-analysis detected this is an Objective-C/C++ file.
+                    # We use both C/C++ and Objective-C/C++ as Objective-C/C++ files are few but share most characteristics
+                    # with C/C++ files: we don't want to lose this information by just overwriting the type, but we want
+                    # the additional information that it is an Objective-C/C++ file.
+                    if type_ == "C/C++" and (
+                        metrics.get("language") == "obj-c/c++" or ext in {".m", ".mm"}
+                    ):
+                        commit.types.add("Objective-C/C++")
 
             commit.types.add(type_)
         else:
@@ -517,34 +553,30 @@ def _transform(commit):
 
     if metrics_file_count:
         commit.average_cyclomatic = commit.total_cyclomatic / metrics_file_count
-        commit.average_halstead_unique_operands = (
-            commit.total_halstead_unique_operands / metrics_file_count
-        )
-        commit.average_halstead_operands = (
-            commit.total_halstead_operands / metrics_file_count
-        )
-        commit.average_halstead_unique_operators = (
-            commit.total_halstead_unique_operators / metrics_file_count
-        )
-        commit.average_halstead_operators = (
-            commit.total_halstead_operators / metrics_file_count
-        )
+        commit.average_halstead_n2 = commit.total_halstead_n2 / metrics_file_count
+        commit.average_halstead_N2 = commit.total_halstead_N2 / metrics_file_count
+        commit.average_halstead_n1 = commit.total_halstead_n1 / metrics_file_count
+        commit.average_halstead_N1 = commit.total_halstead_N1 / metrics_file_count
         commit.average_source_loc = commit.total_source_loc / metrics_file_count
         commit.average_logical_loc = commit.total_logical_loc / metrics_file_count
     else:
         # these values are initialized with sys.maxsize (because we take the min)
         # if no files, then reset them to 0 (it'd be stupid to have min > max)
         commit.minimum_cyclomatic = 0
-        commit.minimum_halstead_operands = 0
-        commit.minimum_halstead_unique_operands = 0
-        commit.minimum_halstead_operators = 0
-        commit.minimum_halstead_unique_operators = 0
+        commit.minimum_halstead_N2 = 0
+        commit.minimum_halstead_n2 = 0
+        commit.minimum_halstead_N1 = 0
+        commit.minimum_halstead_n1 = 0
         commit.minimum_source_loc = 0
         commit.minimum_logical_loc = 0
 
     return commit
 
 
+def _transform(commit):
+    return transform(HG, REPO_DIR, commit)
+
+
 def hg_log(hg, revs):
     template = "{node}\\0{author}\\0{desc}\\0{date|hgdate}\\0{bug}\\0{backedoutby}\\0{author|email}\\0{pushdate|hgdate}\\0{reviewers}\\0"
 
@@ -600,7 +632,7 @@ def _hg_log(revs):
 
 
 def get_revs(hg, rev_start=0, rev_end="tip"):
-    print(f"Getting revs from {rev_start} to {rev_end}...")
+    logger.info(f"Getting revs from {rev_start} to {rev_end}...")
 
     args = hglib.util.cmdbuilder(
         b"log",
@@ -616,11 +648,19 @@ def get_revs(hg, rev_start=0, rev_end="tip"):
 class Experiences:
     def __init__(self, save):
         self.save = save
-        self.db_experiences = shelve.Shelf(
-            LMDBDict("data/commit_experiences.lmdb"),
-            protocol=pickle.DEFAULT_PROTOCOL,
-            writeback=save,
-        )
+
+        try:
+            self.db_experiences = shelve.Shelf(
+                LMDBDict("data/commit_experiences.lmdb", readonly=not save),
+                protocol=pickle.DEFAULT_PROTOCOL,
+                writeback=save,
+            )
+        except lmdb.Error as e:
+            if not save and "No such file or directory" in str(e):
+                self.db_experiences = {}
+            else:
+                raise
+
         if not save:
             self.mem_experiences = {}
 
@@ -648,7 +688,7 @@ class Experiences:
 
 
 def calculate_experiences(commits, first_pushdate, save=True):
-    print(f"Analyzing experiences from {len(commits)} commits...")
+    logger.info(f"Analyzing seniorities from {len(commits)} commits...")
 
     experiences = Experiences(save)
 
@@ -661,6 +701,8 @@ def calculate_experiences(commits, first_pushdate, save=True):
             time_lapse = commit.pushdate - experiences[key]
             commit.seniority_author = time_lapse.total_seconds()
 
+    logger.info(f"Analyzing experiences from {len(commits)} commits...")
+
     # Note: In the case of files, directories, components, we can't just use the sum of previous commits, as we could end
     # up overcounting them. For example, consider a commit A which modifies "dir1" and "dir2", a commit B which modifies
     # "dir1" and a commit C which modifies "dir1" and "dir2". The number of previous commits touching the same directories
@@ -673,23 +715,18 @@ def calculate_experiences(commits, first_pushdate, save=True):
         key = get_key(exp_type, commit_type, item)
         if key not in experiences:
             experiences[key] = utils.ExpQueue(day, EXPERIENCE_TIMESPAN + 1, default)
-        return experiences[key][day]
-
-    def set_experience(exp_type, commit_type, item, day, val):
-        experiences[get_key(exp_type, commit_type, item)][day] = val
+        return experiences[key]
 
     def update_experiences(experience_type, day, items):
         for commit_type in ["", "backout"]:
-            total_exps = [
+            exp_queues = [
                 get_experience(experience_type, commit_type, item, day, 0)
                 for item in items
             ]
+            total_exps = [exp_queues[i][day] for i in range(len(items))]
             timespan_exps = [
-                exp
-                - get_experience(
-                    experience_type, commit_type, item, day - EXPERIENCE_TIMESPAN, 0
-                )
-                for exp, item in zip(total_exps, items)
+                exp - exp_queues[i][day - EXPERIENCE_TIMESPAN]
+                for exp, i in zip(total_exps, range(len(items)))
             ]
 
             total_exps_sum = sum(total_exps)
@@ -719,26 +756,18 @@ def calculate_experiences(commits, first_pushdate, save=True):
                 or commit_type == "backout"
                 and commit.ever_backedout
             ):
-                for i, item in enumerate(items):
-                    set_experience(
-                        experience_type, commit_type, item, day, total_exps[i] + 1
-                    )
+                for i in range(len(items)):
+                    exp_queues[i][day] = total_exps[i] + 1
 
     def update_complex_experiences(experience_type, day, items):
         for commit_type in ["", "backout"]:
-            all_commit_lists = [
+            exp_queues = [
                 get_experience(experience_type, commit_type, item, day, tuple())
                 for item in items
             ]
+            all_commit_lists = [exp_queues[i][day] for i in range(len(items))]
             before_commit_lists = [
-                get_experience(
-                    experience_type,
-                    commit_type,
-                    item,
-                    day - EXPERIENCE_TIMESPAN,
-                    tuple(),
-                )
-                for item in items
+                exp_queues[i][day - EXPERIENCE_TIMESPAN] for i in range(len(items))
             ]
             timespan_commit_lists = [
                 commit_list[len(before_commit_list) :]
@@ -792,14 +821,8 @@ def calculate_experiences(commits, first_pushdate, save=True):
                 or commit_type == "backout"
                 and commit.ever_backedout
             ):
-                for i, item in enumerate(items):
-                    set_experience(
-                        experience_type,
-                        commit_type,
-                        item,
-                        day,
-                        all_commit_lists[i] + (commit.node,),
-                    )
+                for i in range(len(items)):
+                    exp_queues[i][day] = all_commit_lists[i] + (commit.node,)
 
     # prev_day = 0
     # prev_commit = None
@@ -823,7 +846,7 @@ def calculate_experiences(commits, first_pushdate, save=True):
                         experiences[orig_key]
                     )
                 else:
-                    print(
+                    logger.warning(
                         f"Experience missing for file {orig}, type '{commit_type}', on commit {commit.node}"
                     )
 
@@ -836,12 +859,14 @@ def calculate_experiences(commits, first_pushdate, save=True):
             update_complex_experiences("component", day, commit.components)
 
 
-def set_commits_to_ignore(repo_dir, commits):
+def set_commits_to_ignore(hg, repo_dir, commits, ignore_no_bug=True):
     # Skip commits which are in .hg-annotate-ignore-revs or which have
     # 'ignore-this-changeset' in their description (mostly consisting of very
     # large and not meaningful formatting changes).
-    with open(os.path.join(repo_dir, ".hg-annotate-ignore-revs"), "r") as f:
-        ignore_revs = set(l[:40] for l in f)
+    ignore_revs_content = hg.cat(
+        [os.path.join(repo_dir, ".hg-annotate-ignore-revs").encode("ascii")], rev=b"-1"
+    ).decode("utf-8")
+    ignore_revs = set(l[:40] for l in ignore_revs_content.splitlines())
 
     backouts = set(commit.backedoutby for commit in commits if commit.ever_backedout)
 
@@ -854,7 +879,7 @@ def set_commits_to_ignore(repo_dir, commits):
             return True
 
         # Don't analyze commits that are not linked to a bug.
-        if commit.bug_id is None:
+        if ignore_no_bug and commit.bug_id is None:
             return True
 
         return False
@@ -864,7 +889,7 @@ def set_commits_to_ignore(repo_dir, commits):
 
 
 def download_component_mapping():
-    global path_to_component
+    path_to_component = get_component_mapping(False)
 
     utils.download_check_etag(
         "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/gecko.v2.mozilla-central.latest.source.source-bugzilla-info/artifacts/public/components.json",
@@ -872,20 +897,30 @@ def download_component_mapping():
     )
 
     with open("data/component_mapping.json", "r") as f:
-        path_to_component = json.load(f)
+        data = json.load(f)
 
-    path_to_component = {
-        path: "::".join(component) for path, component in path_to_component.items()
-    }
+    for path, component in data.items():
+        path_to_component[path.encode("utf-8")] = "::".join(component).encode("utf-8")
+
+    close_component_mapping()
+
+
+def get_component_mapping(readonly=True):
+    global path_to_component
+    path_to_component = LMDBDict("data/component_mapping.lmdb", readonly=readonly)
+    return path_to_component
+
+
+def close_component_mapping():
+    global path_to_component
+    path_to_component.close()
+    path_to_component = None
 
 
 def hg_log_multi(repo_dir, revs):
     if len(revs) == 0:
         return []
 
-    cwd = os.getcwd()
-    os.chdir(repo_dir)
-
     threads_num = os.cpu_count() + 1
     REVS_COUNT = len(revs)
     CHUNK_SIZE = int(math.ceil(REVS_COUNT / threads_num))
@@ -895,14 +930,12 @@ def hg_log_multi(repo_dir, revs):
     ]
 
     with concurrent.futures.ThreadPoolExecutor(
-        initializer=_init_thread, max_workers=threads_num
+        initializer=_init_thread, initargs=(repo_dir,), max_workers=threads_num
     ) as executor:
         commits = executor.map(_hg_log, revs_groups)
         commits = tqdm(commits, total=len(revs_groups))
         commits = list(itertools.chain.from_iterable(commits))
 
-    os.chdir(cwd)
-
     while len(hg_servers) > 0:
         hg_server = hg_servers.pop()
         hg_server.close()
@@ -910,48 +943,75 @@ def hg_log_multi(repo_dir, revs):
     return commits
 
 
-def download_commits(repo_dir, rev_start=0, save=True):
+@lru_cache(maxsize=None)
+def get_first_pushdate(repo_dir):
     with hglib.open(repo_dir) as hg:
-        revs = get_revs(hg, rev_start)
-        if len(revs) == 0:
-            print("No commits to analyze")
-            return []
+        return hg_log(hg, [b"0"])[0].pushdate
 
-        first_pushdate = hg_log(hg, [b"0"])[0].pushdate
 
-    print(f"Mining {len(revs)} commits using {os.cpu_count()} processes...")
+def download_commits(
+    repo_dir,
+    rev_start=None,
+    revs=None,
+    save=True,
+    use_single_process=False,
+    ignore_no_bug=True,
+):
+    assert revs is not None or rev_start is not None
 
-    commits = hg_log_multi(repo_dir, revs)
+    with hglib.open(repo_dir) as hg:
+        if revs is None:
+            revs = get_revs(hg, rev_start)
+            if len(revs) == 0:
+                logger.info("No commits to analyze")
+                return []
 
-    print("Downloading file->component mapping...")
+        first_pushdate = get_first_pushdate(repo_dir)
 
-    download_component_mapping()
+        logger.info(f"Mining {len(revs)} commits...")
 
-    set_commits_to_ignore(repo_dir, commits)
+        if not use_single_process:
+            logger.info(f"Using {os.cpu_count()} processes...")
+            commits = hg_log_multi(repo_dir, revs)
+        else:
+            commits = hg_log(hg, revs)
 
-    commits_num = len(commits)
+        if save or not os.path.exists("data/component_mapping.lmdb"):
+            logger.info("Downloading file->component mapping...")
+            download_component_mapping()
 
-    print(f"Mining {commits_num} commits using {os.cpu_count()} processes...")
+        set_commits_to_ignore(hg, repo_dir, commits, ignore_no_bug)
 
-    global rs_parsepatch
-    import rs_parsepatch
+        commits_num = len(commits)
 
-    from bugbug import rust_code_analysis_server
+        logger.info(f"Mining {commits_num} patches...")
 
-    global code_analysis_server
-    code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer()
+        global rs_parsepatch
+        import rs_parsepatch
 
-    with concurrent.futures.ProcessPoolExecutor(
-        initializer=_init, initargs=(repo_dir,)
-    ) as executor:
-        commits = executor.map(_transform, commits, chunksize=64)
-        commits = tqdm(commits, total=commits_num)
-        commits = list(commits)
+        global code_analysis_server
+        code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer()
+
+        if not use_single_process:
+            with concurrent.futures.ProcessPoolExecutor(
+                initializer=_init_process, initargs=(repo_dir,)
+            ) as executor:
+                commits = executor.map(_transform, commits, chunksize=64)
+                commits = tqdm(commits, total=commits_num)
+                commits = list(commits)
+        else:
+            get_component_mapping()
+
+            commits = [transform(hg, repo_dir, c) for c in commits]
+
+            close_component_mapping()
 
     code_analysis_server.terminate()
 
     calculate_experiences(commits, first_pushdate, save)
 
+    logger.info(f"Applying final commits filtering...")
+
     commits = [commit.to_dict() for commit in commits if not commit.ignored]
 
     if save:
@@ -960,30 +1020,36 @@ def download_commits(repo_dir, rev_start=0, save=True):
     return commits
 
 
-def clean(repo_dir, pull=True):
-    with hglib.open(repo_dir) as hg:
-        hg.revert(repo_dir.encode("utf-8"), all=True)
+def clean(hg, repo_dir, pull=True):
+    logger.info("Restoring files to their checkout state...")
+    hg.revert(repo_dir.encode("utf-8"), all=True)
 
-        try:
-            cmd = hglib.util.cmdbuilder(
-                b"strip", rev=b"roots(outgoing())", force=True, backup=False
-            )
-            hg.rawcommand(cmd)
-        except hglib.error.CommandError as e:
-            if b"abort: empty revision set" not in e.err:
-                raise
+    logger.info("Stripping non-public commits...")
+    try:
+        cmd = hglib.util.cmdbuilder(
+            b"strip", rev=b"roots(outgoing())", force=True, backup=False
+        )
+        hg.rawcommand(cmd)
+    except hglib.error.CommandError as e:
+        if b"abort: empty revision set" not in e.err:
+            raise
 
-        # Pull and update.
-        if pull:
-            logger.info(f"Pulling and updating {repo_dir}")
-            hg.pull(update=True)
-            logger.info(f"{repo_dir} pulled and updated")
+    # Pull and update.
+    if pull:
+        logger.info(f"Pulling and updating {repo_dir}")
+        hg.pull(update=True)
+        logger.info(f"{repo_dir} pulled and updated")
 
 
-def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central"):
-    if os.path.exists(repo_dir):
-        clean(repo_dir)
+def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
+    try:
+        with hglib.open(repo_dir) as hg:
+            clean(hg, repo_dir)
+
         return
+    except hglib.error.ServerError as e:
+        if "abort: repository" not in str(e) and b"not found" not in str(e):
+            raise
 
     cmd = hglib.util.cmdbuilder(
         "robustcheckout",
@@ -993,6 +1059,7 @@ def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central"):
         sharebase=repo_dir + "-shared",
         networkattempts=7,
         branch=b"tip",
+        noupdate=not update,
     )
 
     cmd.insert(0, hglib.HGPATH)
@@ -1008,83 +1075,20 @@ def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central"):
         logger.info("pushlog database doesn't exist")
 
     # Pull and update, to make sure the pushlog is generated.
-    clean(repo_dir)
+    with hglib.open(repo_dir) as hg:
+        clean(hg, repo_dir)
 
 
 def apply_stack(repo_dir, stack, branch):
     """Apply a stack of patches on a repository"""
     assert len(stack) > 0, "Empty stack"
 
-    def has_revision(revision):
-        try:
-            hg.identify(revision)
-            return True
-        except hglib.error.CommandError:
-            return False
-
-    def apply_patches(base, patches):
-
-        # Update to base revision
-        logger.info(f"Updating {repo_dir} to {base}")
-        hg.update(base, clean=True)
-
-        # Then apply each patch in the stack
-        try:
-            for node, patch in patches:
-                hg.import_(patches=io.BytesIO(patch.encode("utf-8")), user="bugbug")
-        except hglib.error.CommandError as e:
-            logger.warning(f"Failed to apply patch {node}: {e.err}")
-            return False
-        return True
-
-    # Find the base revision to apply all the patches onto
-    # Use first parent from first patch if all its parents are available
-    # Otherwise fallback on tip
-    def get_base():
-        parents = stack[0]["parents"]
-        assert len(parents) > 0, "No parents found for first patch"
-        if all(map(has_revision, parents)):
-            return parents[0]
-
-        return "tip"
-
-    def first_in_stack():
-        return hg.log(revrange=f"-{len(stack)}")[0].node.decode("utf-8")
-
-    # Start by cleaning the repo, without pulling
-    clean(repo_dir, pull=False)
-
     with hglib.open(repo_dir) as hg:
-        # Get initial base revision
-        base = get_base()
-
-        # Load all the patches in the stack
-        patches = [(rev["node"], get_hgmo_patch(branch, rev["node"])) for rev in stack]
-        logger.info(f"Loaded {len(patches)} patches for the stack")
-
-        # Apply all the patches in the stack on current base
-        if apply_patches(base, patches):
-            logger.info(f"Stack applied successfully on {base}")
-            return first_in_stack()
-
-        # We tried to apply on the valid parent and failed: cannot try another revision
-        if base != "tip":
-            raise Exception(f"Failed to apply on valid parent {base}")
-
-        # We tried to apply on tip, let's try to find the valid parent after pulling
-        clean(repo_dir, pull=True)
-
-        # Check if the valid base is now available
-        new_base = get_base()
-        if base == new_base:
-            raise Exception("No valid parent found for the stack")
-
-        if not apply_patches(new_base, patches):
-            raise Exception("Failed to apply stack on second try")
-
-        logger.info(f"Stack applied successfully on {new_base}")
-
-        return first_in_stack()
+        hg.pull(
+            source=f"https://hg.mozilla.org/{branch}/".encode("ascii"),
+            rev=stack[-1]["pushhead"].encode("ascii"),
+        )
+        return [rev["node"].encode("ascii") for rev in stack]
 
 
 if __name__ == "__main__":
@@ -1095,4 +1099,4 @@ if __name__ == "__main__":
     )
     args = parser.parse_args()
 
-    download_commits(args.repository_dir, args.rev_start)
+    download_commits(args.repository_dir, args.rev_start, save=False)
