@@ -4,13 +4,16 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import concurrent.futures
+import errno
 import json
 import logging
 import os
 import socket
+import subprocess
 import tarfile
 from collections import deque
 from contextlib import contextmanager
+from functools import lru_cache
 
 import boto3
 import dateutil.parser
@@ -21,6 +24,7 @@ import scipy
 import taskcluster
 import zstandard
 from pkg_resources import DistributionNotFound
+from requests.packages.urllib3.util.retry import Retry
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.compose import ColumnTransformer
 from sklearn.preprocessing import OrdinalEncoder
@@ -144,10 +148,14 @@ def get_secret(secret_id):
         raise ValueError("Failed to find secret {}".format(secret_id))
 
 
-def upload_s3(paths):
+def get_s3_credentials():
     auth = taskcluster.Auth(get_taskcluster_options())
     response = auth.awsS3Credentials("read-write", "communitytc-bugbug", "data/")
-    credentials = response["credentials"]
+    return response["credentials"]
+
+
+def upload_s3(paths):
+    credentials = get_s3_credentials()
 
     client = boto3.client(
         "s3",
@@ -215,6 +223,7 @@ def download_model(model_name):
     updated = download_check_etag(url)
     if updated:
         zstd_decompress(path)
+        os.remove(f"{path}.zst")
     assert os.path.exists(path), "Decompressed file exists"
     return path
 
@@ -225,7 +234,7 @@ def download_and_load_model(model_name):
 
 
 def zstd_compress(path):
-    cctx = zstandard.ZstdCompressor()
+    cctx = zstandard.ZstdCompressor(threads=-1)
     with open(path, "rb") as input_f:
         with open(f"{path}.zst", "wb") as output_f:
             cctx.copy_stream(input_f, output_f)
@@ -239,20 +248,49 @@ def zstd_decompress(path):
 
 
 @contextmanager
-def open_tar_zst(path):
-    cctx = zstandard.ZstdCompressor()
-    with open(path, "wb") as f:
-        with cctx.stream_writer(f) as compressor:
-            with tarfile.open(mode="w|", fileobj=compressor) as tar:
-                yield tar
+def open_tar_zst(path, mode):
+    if mode == "w":
+        cctx = zstandard.ZstdCompressor(threads=-1)
+        with open(path, f"wb") as f:
+            with cctx.stream_writer(f) as compressor:
+                with tarfile.open(mode=f"w|", fileobj=compressor) as tar:
+                    yield tar
+    elif mode == "r":
+        dctx = zstandard.ZstdDecompressor()
+        with open(path, "rb") as f:
+            with dctx.stream_reader(f) as reader:
+                with tarfile.open(mode="r|", fileobj=reader) as tar:
+                    yield tar
+    else:
+        assert False, f"Unexpected mode: {mode}"
 
 
-def extract_tar_zst(path):
-    dctx = zstandard.ZstdDecompressor()
-    with open(f"{path}.zst", "rb") as f:
-        with dctx.stream_reader(f) as reader:
-            with tarfile.open(mode="r|", fileobj=reader) as tar:
-                tar.extractall()
+# Using tar directly is twice as fast than through Python!
+def create_tar_zst(path: str) -> None:
+    inner_path = path[: -len(".tar.zst")]
+
+    if not os.path.exists(inner_path):
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), inner_path)
+
+    subprocess.run(["tar", "-I", "zstd", "-cf", path, inner_path], check=True)
+
+
+def extract_tar_zst(path: str) -> None:
+    if not os.path.exists(path):
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
+
+    subprocess.run(["tar", "-I", "zstd", "-xf", path], check=True)
+
+
+def extract_file(path: str) -> None:
+    inner_path, _ = os.path.splitext(path)
+
+    if str(path).endswith(".tar.zst"):
+        extract_tar_zst(path)
+    elif str(path).endswith(".zst"):
+        zstd_decompress(inner_path)
+    else:
+        assert False, f"Unexpected compression type for {path}"
 
 
 class CustomJsonEncoder(json.JSONEncoder):
@@ -322,22 +360,32 @@ class ExpQueue:
 
 
 class LMDBDict:
-    def __init__(self, path):
+    def __init__(self, path, readonly=False):
+        self.readonly = readonly
         self.db = lmdb.open(
-            path, map_size=68719476736, metasync=False, sync=False, meminit=False
+            path,
+            map_size=68719476736,
+            metasync=False,
+            sync=False,
+            meminit=False,
+            readonly=readonly,
         )
-        self.txn = self.db.begin(buffers=True, write=True)
+        self.txn = self.db.begin(buffers=True, write=not readonly)
 
     def close(self):
         self.txn.commit()
-        self.db.sync()
+        if not self.readonly:
+            self.db.sync()
         self.db.close()
 
     def __contains__(self, key):
         return self.txn.get(key) is not None
 
     def __getitem__(self, key):
-        return self.txn.get(key)
+        val = self.txn.get(key)
+        if val is None:
+            raise KeyError
+        return val
 
     def __setitem__(self, key, value):
         self.txn.put(key, value, dupdata=False)
@@ -372,9 +420,48 @@ class ThreadPoolExecutorResult(concurrent.futures.ThreadPoolExecutor):
         return super(ThreadPoolExecutorResult, self).__exit__(*args)
 
 
-def get_hgmo_patch(branch: str, revision: str) -> str:
-    """Load a patch for a given revision"""
-    url = f"https://hg.mozilla.org/{branch}/raw-rev/{revision}"
-    r = requests.get(url)
+@lru_cache(maxsize=None)
+def get_session(name):
+    session = requests.Session()
+
+    retry = Retry(total=9, backoff_factor=1, status_forcelist=[429, 500, 502, 503, 504])
+
+    # Default HTTPAdapter uses 10 connections. Mount custom adapter to increase
+    # that limit. Connections are established as needed, so using a large value
+    # should not negatively impact performance.
+    http_adapter = requests.adapters.HTTPAdapter(
+        pool_connections=50, pool_maxsize=50, max_retries=retry
+    )
+    session.mount("https://", http_adapter)
+    session.mount("http://", http_adapter)
+
+    return session
+
+
+def get_hgmo_stack(branch: str, revision: str) -> list:
+    """Load descriptions of patches in the stack for a given revision"""
+    url = f"https://hg.mozilla.org/{branch}/json-automationrelevance/{revision}"
+    r = get_session("hgmo").get(url)
     r.raise_for_status()
-    return r.text
+
+    def should_skip(changeset):
+        # Analyze all changesets if we are not on try.
+        if branch != "try":
+            return False
+
+        # Don't analyze public changesets, as they are not relevant for a try
+        # push.
+        if changeset["phase"] != "draft":
+            return True
+
+        # Don't analyze empty changesets.
+        if len(changeset["files"]) == 0:
+            return True
+
+        # Don't analyze changesets which simply specify try parameters.
+        if changeset["files"] == ["try_task_config.json"]:
+            return True
+
+        return False
+
+    return [c for c in r.json()["changesets"] if not should_skip(c)]
