@@ -25,7 +25,7 @@ from libmozdata import vcs_map
 from libmozdata.phabricator import PhabricatorAPI
 from scipy.stats import spearmanr
 
-from bugbug import commit_features, db, repository, test_scheduling
+from bugbug import db, repository, test_scheduling
 from bugbug.utils import (
     download_and_load_model,
     download_check_etag,
@@ -134,7 +134,9 @@ class CommitClassifier(object):
 
         self.git_repo_dir = git_repo_dir
         if git_repo_dir:
-            self.clone_git_repo("https://github.com/mozilla/gecko-dev", git_repo_dir)
+            self.clone_git_repo(
+                "hg::https://hg.mozilla.org/mozilla-central", git_repo_dir
+            )
 
         self.method_defect_predictor_dir = method_defect_predictor_dir
         if method_defect_predictor_dir:
@@ -186,48 +188,43 @@ class CommitClassifier(object):
             self.testfailure_model = download_and_load_model("testfailure")
             assert self.testfailure_model is not None
 
-    def clone_git_repo(self, repo_url, repo_dir, rev="master"):
+    def clone_git_repo(self, repo_url, repo_dir, rev="origin/branches/default/tip"):
         logger.info(f"Cloning {repo_url}...")
 
         if not os.path.exists(repo_dir):
             tenacity.retry(
+                wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
+                stop=tenacity.stop_after_attempt(5),
+            )(
                 lambda: subprocess.run(
                     ["git", "clone", "--quiet", repo_url, repo_dir], check=True
-                ),
-                wait=tenacity.wait_fixed(30),
-                stop=tenacity.stop_after_attempt(5),
+                )
             )()
 
         tenacity.retry(
-            lambda: subprocess.run(
-                ["git", "pull", "--quiet", repo_url, "master"],
-                cwd=repo_dir,
-                capture_output=True,
-                check=True,
-            ),
-            wait=tenacity.wait_fixed(30),
+            wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
             stop=tenacity.stop_after_attempt(5),
+        )(
+            lambda: subprocess.run(
+                ["git", "fetch"], cwd=repo_dir, capture_output=True, check=True,
+            )
         )()
 
-        tenacity.retry(
-            lambda: subprocess.run(
-                ["git", "checkout", rev], cwd=repo_dir, capture_output=True, check=True
-            ),
-            wait=tenacity.wait_fixed(30),
-            stop=tenacity.stop_after_attempt(5),
-        )()
+        subprocess.run(
+            ["git", "checkout", rev], cwd=repo_dir, capture_output=True, check=True
+        )
 
     def update_commit_db(self):
-        repository.clone(self.repo_dir)
+        repository.clone(self.repo_dir, update=True)
 
         assert db.download(repository.COMMITS_DB, support_files_too=True)
 
         for commit in repository.get_commits():
             pass
 
-        rev_start = "children({})".format(commit["node"])
-
-        repository.download_commits(self.repo_dir, rev_start)
+        repository.download_commits(
+            self.repo_dir, rev_start="children({})".format(commit["node"])
+        )
 
     def has_revision(self, hg, revision):
         if not revision:
@@ -610,15 +607,8 @@ class CommitClassifier(object):
 
         logger.info(f"Test failure risk: {testfailure_probs[0][1]}")
 
-        commit_data = commit_features.merge_commits(commits)
-
-        push_num = self.past_failures_data["push_num"]
-
-        # XXX: Consider using mozilla-central built-in rules to filter some of these out, e.g. SCHEDULES.
-        all_tasks = self.past_failures_data["all_runnables"]
-
         if not runnable_jobs_path:
-            runnable_jobs = {task for task in all_tasks}
+            runnable_jobs = {}
         elif runnable_jobs_path.startswith("http"):
             r = requests.get(runnable_jobs_path)
             r.raise_for_status()
@@ -627,31 +617,19 @@ class CommitClassifier(object):
             with open(runnable_jobs_path, "r") as f:
                 runnable_jobs = json.load(f)
 
-        # XXX: For now, only restrict to linux64 test tasks.
-        all_tasks = [
-            t
-            for t in all_tasks
-            if t.startswith("test-linux1804-64/") and "test-verify" not in t
-        ]
-
         # XXX: Remove tasks which are not in runnable jobs right away, so we avoid classifying them.
+        # XXX: Consider using mozilla-central built-in rules to filter some of the tasks out, e.g. SCHEDULES.
 
-        commit_tests = []
-        for data in test_scheduling.generate_data(
-            self.past_failures_data, commit_data, push_num, all_tasks, [], []
-        ):
-            if not data["name"].startswith("test-"):
-                continue
+        selected_tasks = list(
+            self.model.select_tests(
+                commits, float(get_secret("TEST_SELECTION_CONFIDENCE_THRESHOLD"))
+            ).values()
+        )
 
-            commit_test = commit_data.copy()
-            commit_test["test_job"] = data
-            commit_tests.append(commit_test)
-
-        probs = self.model.classify(commit_tests, probabilities=True)
-        selected_indexes = np.argwhere(
-            probs[:, 1] > float(get_secret("TEST_SELECTION_CONFIDENCE_THRESHOLD"))
-        )[:, 0]
-        selected_tasks = [commit_tests[i]["test_job"]["name"] for i in selected_indexes]
+        # XXX: For now, only restrict to linux64 test tasks (as for runnable jobs above, we could remove these right away).
+        selected_tasks = [
+            t for t in selected_tasks if t.startswith("test-linux1804-64/")
+        ]
 
         with open("failure_risk", "w") as f:
             f.write(
@@ -662,26 +640,27 @@ class CommitClassifier(object):
             )
 
         # This should be kept in sync with the test scheduling history retriever script.
-        cleaned_selected_tasks = []
-        for selected_task in selected_tasks:
-            if (
-                selected_task.startswith("test-linux64")
-                and selected_task not in runnable_jobs
-            ):
-                selected_task = selected_task.replace(
-                    "test-linux64-", "test-linux1804-64-"
-                )
+        if len(runnable_jobs) > 0:
+            cleaned_selected_tasks = []
+            for selected_task in selected_tasks:
+                if (
+                    selected_task.startswith("test-linux64")
+                    and selected_task not in runnable_jobs
+                ):
+                    selected_task = selected_task.replace(
+                        "test-linux64-", "test-linux1804-64-"
+                    )
 
-            if (
-                selected_task.startswith("test-linux1804-64-")
-                and selected_task not in runnable_jobs
-            ):
-                selected_task = selected_task.replace(
-                    "test-linux1804-64-", "test-linux64-"
-                )
+                if (
+                    selected_task.startswith("test-linux1804-64-")
+                    and selected_task not in runnable_jobs
+                ):
+                    selected_task = selected_task.replace(
+                        "test-linux1804-64-", "test-linux64-"
+                    )
 
-            if selected_task in runnable_jobs:
-                cleaned_selected_tasks.append(selected_task)
+                if selected_task in runnable_jobs:
+                    cleaned_selected_tasks.append(selected_task)
 
         # It isn't worth running the build associated to the tests, if we only run three test tasks.
         if len(cleaned_selected_tasks) < 3:
