@@ -47,7 +47,7 @@ IGNORED_COMMITS_DB = "data/ignored_commits.json"
 db.register(
     IGNORED_COMMITS_DB,
     "https://s3-us-west-2.amazonaws.com/communitytc-bugbug/data/ignored_commits.json.zst",
-    1,
+    2,
 )
 
 
@@ -95,35 +95,33 @@ class RegressorFinder(object):
     def clone_git_repo(self, repo_url, repo_dir):
         if not os.path.exists(repo_dir):
             tenacity.retry(
+                wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
+                stop=tenacity.stop_after_attempt(5),
+            )(
                 lambda: subprocess.run(
                     ["git", "clone", "--quiet", repo_url, repo_dir], check=True
-                ),
-                wait=tenacity.wait_fixed(30),
-                stop=tenacity.stop_after_attempt(5),
+                )
             )()
 
             logger.info(f"{repo_dir} cloned")
 
-        logger.info(f"Pulling and updating {repo_dir}")
+        logger.info(f"Fetching {repo_dir}")
 
         tenacity.retry(
+            wait=tenacity.wait_exponential(multiplier=1, min=16, max=64),
+            stop=tenacity.stop_after_attempt(5),
+        )(
             lambda: subprocess.run(
-                ["git", "pull", "--quiet", repo_url, "master"],
+                ["git", "fetch", "--quiet"],
                 cwd=repo_dir,
                 capture_output=True,
                 check=True,
-            ),
-            wait=tenacity.wait_fixed(30),
-            stop=tenacity.stop_after_attempt(5),
+            )
         )()
 
-        logger.info(f"{repo_dir} pulled and updated")
+        logger.info(f"{repo_dir} fetched")
 
     def init_mapping(self):
-        logger.info("Downloading Mercurial <-> git mapping file...")
-        vcs_map.download_mapfile()
-        vcs_map.load_mapfile()
-
         if self.tokenized_git_repo_url is not None:
             (
                 self.tokenized_git_to_mercurial,
@@ -154,20 +152,13 @@ class RegressorFinder(object):
         with hglib.open(self.mercurial_repo_dir) as hg:
             revs = repository.get_revs(hg, rev_start)
 
-        # Drop commits which are not yet present in the mercurial <-> git mapping.
-        while len(revs) > 0:
-            try:
-                vcs_map.mercurial_to_git(revs[-1].decode("ascii"))
-                break
-            except Exception as e:
-                if not str(e).startswith("Missing mercurial commit in the VCS map"):
-                    raise
-
-                revs.pop()
-
         commits = repository.hg_log_multi(self.mercurial_repo_dir, revs)
 
-        repository.set_commits_to_ignore(self.mercurial_repo_dir, commits)
+        with hglib.open(self.mercurial_repo_dir) as hg:
+            repository.set_commits_to_ignore(hg, self.mercurial_repo_dir, commits)
+
+        for commit in commits:
+            commit.ignored |= commit.author_email == "wptsync@mozilla.com"
 
         chosen_commits = set()
         commits_to_ignore = []
@@ -319,13 +310,13 @@ class RegressorFinder(object):
             if tokenized:
                 return self.tokenized_git_to_mercurial[rev]
             else:
-                return vcs_map.git_to_mercurial(rev)
+                return vcs_map.git_to_mercurial(repo_dir, rev)
 
         def mercurial_to_git(rev):
             if tokenized:
                 return self.mercurial_to_tokenized_git[rev]
             else:
-                return vcs_map.mercurial_to_git(rev)
+                return vcs_map.mercurial_to_git(repo_dir, rev)
 
         logger.info("Download previously found bug-introducing commits...")
         db.download(db_path)
